@@ -41,7 +41,7 @@ plot.mdsbi <- function(x, vecscale = NULL, plot.dim = c(1,2), sphere = TRUE, col
   plot(x$conf[,x1], x$conf[,y1], main = main, type = type, xlab = xlab, ylab = ylab, 
        xlim = xlim, ylim = ylim, pch = pch, asp = asp, col = col, ...)
   if (label.conf$label) {
-    if (label.conf$pos == 5) {
+    if (label.conf$pos[1] == 5) {
       thigmophobe.labels(x$conf[,x1], x$conf[,y1], labels = rownames(x$conf), 
                          cex = label.conf$cex, text.pos = NULL, 
                          col = label.conf$col)  
